"use strict";
/* global browser, tldjs */


const NetworkObserver = {
  resolvesCanaryDomains: false,
  usesSafeSearch: false,
  notInPSL: new Set(),
  pslResolvesLocal: new Set(),
  uniqueDomains: new Set(),
};


// TODO: Randomize order of lookups
async function dnsListLookup(domainList) {
  let results = [];
  for (let i = 0; i < domainList.length; i++) {
    let domain = domainList[i];
    try {
      let flags = ["disable_trr", "disable_ipv6", "bypass_cache"];
      let response = await browser.dns.resolve(domain, flags);
      results = results.concat(response.addresses);
    } catch (e) {
      // Handle NXDOMAIN
      if (e.message === "NS_ERROR_UNKNOWN_HOST") {
        results = results.concat(null);
      } else {
        throw e;
      }
    }
  }
  return results;
}


async function usesSafeSearch() {
  const strictDomains = [
    {
      check: [
        "www.google.com",
        "google.com"
      ],
      filter: [
        "forcesafesearch.google.com",
      ],
    },
    {
      check: [
        "www.bing.com",
        "bing.com"
      ],
      filter: [
        "strict.bing.com"
      ],
    },
    {
      check: [
        "www.youtube.com",
        "m.youtube.com",
        "youtubei.googleapis.com",
        "youtube.googleapis.com",
        "www.youtube-nocookie.com"
      ],
      filter: [
        "restrict.youtube.com",
        "restrictmoderate.youtube.com"
      ],
    }
  ];
  
  // Compare strict domain lookups to non-strict domain lookups
  for (let i = 0; i < strictDomains.length; i++) {
    let provider = strictDomains[i];
    let results = {filtered: false};
    results.checks = await dnsListLookup(provider.check);
    results.filters = await dnsListLookup(provider.filter);

    for (let filter of results.filters) {
      let filtered = results.checks.includes(filter);
      if (filtered) {
        return true;
      }
    }
  }
  return false;
}


async function resolvesCanaryDomains() {
  const canaryDomains = ["testprotectedbrowsing.xfinity.com",
                         "testparentalcontrols.xfinity.com"];
 
  let answers = await dnsListLookup(canaryDomains);
  for (let i = 0; i < answers.length; i++) {
    // If NXDOMAIN is not returned, we infer that content filters are on
    let answer = answers[i];
    if (answer) {
      return true;
    }
  }
  return false;
}


async function resolvesBlocklistDomains() {
  const blocklistDomains = ["exampleadultsite.com"];

  let answers = await dnsListLookup(blocklistDomains);
  for (let i = 0; i < answers.length; i++) {
    // If NXDOMAIN is not returned, we infer that content filters are on
    let answer = answers[i];
    if (answer) {
      return true;
    }
  }
  return false;
}


async function checkContentFilters() {
  let _resolvesCanaryDomains = await resolvesCanaryDomains();
  let _resolvesBlocklistDomains = await resolvesBlocklistDomains();
  let _usesSafeSearch = await usesSafeSearch();
  let contentFilterData= {"resolvesCanaryDomains": _resolvesCanaryDomains,
                          "resolvesBlocklistDomains": _resolvesBlocklistDomains,
                          "usesSafeSearch": _usesSafeSearch};
  await browser.storage.local.set({"contentFilterData": contentFilterData});
}


// Private address space for IPv4
// TODO: Add function to parse local addresses for IPv6 (RFC 4193)
function isRFC1918(ip) {
  if (!ip) {
    return false;
  }

  // Parse the highest order octets of the address
  let octets = ip.split(".");
  let firstOctet = parseInt(octets[0], 10);
  let secondOctet = parseInt(octets[1], 10); 
    
  if (firstOctet === 10) {
    return true;
  } else if (firstOctet === 196 && secondOctet === 168) {
    return true;
  } else if (firstOctet === 173 && secondOctet >= 16  && secondOctet <= 31) {
    return true;
  }
  return false;
}


// TODO: Detect HTTPS upgrades
async function checkSplitHorizon(responseDetails) {
  let _ip = responseDetails.ip;
  let _hostname = new URL(responseDetails.url).hostname;
  if (!_ip || !_hostname || _hostname === "localhost") {
    console.error("Empty/invalid hostname or IP addresses for", 
                  responseDetails.url, _ip, _hostname);
    return;
  }

  // Check if the domain suffix is not in the PSL 
  let _tldExists = tldjs.tldExists(_hostname);
  if (!_tldExists) {
    NetworkObserver.notInPSL.add(_hostname);
  }

  // Check if the domain suffix is in the PSL 
  // but the domain name resolves to an RFC 1918 address
  let _isRFC1918 = isRFC1918(_ip);
  if (_tldExists && _isRFC1918) {
    NetworkObserver.pslResolvesLocal.add(_hostname);
  }
  
  NetworkObserver.uniqueDomains.add(_hostname);
  let splitHorizonData = {"notInPSL": NetworkObserver.notInPSL,
                          "pslResolvesLocal": NetworkObserver.pslResolvesLocal,
                          "uniqueDomains": NetworkObserver.uniqueDomains};
  await browser.storage.local.set({"splitHorizonData": splitHorizonData}); 
}
