"use strict";
/* global browser, checkContentFilters, checkSplitHorizon */


const baseStudySetup = {
  activeExperimentName: browser.runtime.id,
  studyType: "shield",
  // telemetry
  telemetry: {
    // default false. Actually send pings.
    send: true,
    // Marks pings with testing=true.  Set flag to `true` before final release
    removeTestingFlag: false,
  },
  endings: {
    // standard endings
    "user-disable": {
      baseUrls: [],
      category: "ended-negative",
    },
    ineligible: {
      baseUrls: [],
      category: "ended-neutral",
    },
    expired: {
      baseUrls: [],
      category: "ended-positive",
    },
  },
  weightedVariations: [
    {
      name: "study-active",
      weight: 1.5
    }
  ],
  // maximum time that the study should run, from the first run
  expire: {
    days: 7,
  },
  allowEnroll: true,
};


const stateManager = {
  _settingName: null,

  set settingName(settingName) {
    if (this._settingName === null) {
      this._settingName = settingName;
    } else {
      console.error("study setting already set");
    }
  },

  get settingName() {
    if (this._settingName === null) {
      throw new Error("study setting not set");
    } else {
      return this._settingName;
    }
  },

  async getState() {
    return await browser.experiments.settings.get(this.settingName) || null;
  },

  async setState(stateKey) {
    browser.study.sendTelemetry({stateKey});
    browser.experiments.settings.set(this.settingName, stateKey);
  },

  /* settingName impacts the active states file we will be getting:
     study-active
   */
  async setSetting(settingName) {
    stateManager.settingName = settingName;
    return browser.experiments.settings.add(this.settingName);
  },

  endStudy(stateKey) {
    browser.study.sendTelemetry({stateKey, disabling: "yes"});
    browser.study.endStudy(stateKey || "generic");
  },

  // Clear out settings
  async clear(stateKey = null) {
    browser.experiments.settings.clear(stateKey);
  }
};


const rollout = {
  async init() {
    browser.study.onEndStudy.addListener((ending) => {
      stateManager.clear(ending);
    });
    browser.study.onReady.addListener(() => {
      this.onReady();
    });
    await browser.study.setup(baseStudySetup);
  },

  async captivePortalListener(details) {
    console.log("Captive portal unlocked", details);
    stateManager.setState("loaded");
    await this.telemetry();
    await this.measure();
  },

  async onReady() {
    // If the user hasn't met the criteria clean up
    if (await browser.experiments.settings.hasModifiedPrerequisites()) {
      console.log("TRR enabled by user; ineligible for study");
      stateManager.endStudy("ineligible");
      return;
    }

    // Return early as we don't have a control.json file
    const studyInfo = await browser.study.getStudyInfo();
    const variation = studyInfo.variation.name;
    await stateManager.setSetting(variation);
    if (variation === "control") {
      return;
    }
    if (studyInfo.isFirstRun) {
      stateManager.setState("installed");
    }

    await stateManager.setState("loaded");
    await this.telemetry();
    await this.measure();

    // If the captive portal is unlocked, run the measurement
    // let captiveState = await browser.captivePortal.getState();
    // if (captiveState === "unlocked_portal" || captiveState === "not_captive") {
    //   await stateManager.setState("loaded");
    //   await this.telemetry();
    //   await this.measure();
    // }

    // Listen to the captive portal when it unlocks
    // browser.captivePortal.onConnectivityAvailable.addListener(this.telemetry_and_measure);
  },


  // Entry point for sending Telemetry data from local storage
  async telemetry() {
    let cfData = await browser.storage.local.get("contentFilterData");
    let shData = await browser.storage.local.get("splitHorizonData");
    cfData = cfData.contentFilterData;
    shData = shData.splitHorizonData;
    await browser.storage.local.clear();

    // Send Telemetry data
    if (cfData && shData) {
      // Read data from local storage
      let resolvesCanaryDomains = String(cfData.resolvesCanaryDomains);
      let resolvesBlocklistDomains = String(cfData.resolvesBlocklistDomains);
      let usesSafeSearch = String(cfData.usesSafeSearch);
      let notInPSL = String(shData.notInPSL.size);
      let pslResolvesLocal = String(shData.pslResolvesLocal.size);
      let uniqueDomains = String(shData.uniqueDomains.size);
      let telemetryData = {"resolvesCanaryDomains": resolvesCanaryDomains,
                           "resolvesBlocklistDomains": resolvesBlocklistDomains,
                           "usesSafeSearch": usesSafeSearch,
                           "notInPSL": notInPSL,
                           "pslResolvesLocal": pslResolvesLocal,
                           "uniqueDomains": uniqueDomains};

      // Send Telemety data, and check copies of pings on disk
      await browser.study.sendTelemetry(telemetryData);
    }
  },

  // Entry point for running measurements
  async measure() {
    // Check for content filters by: 
    //   a) resolving canary domains 
    //   b) resolving "safe search" domains for YouTube, Google, and Bing
    //   c) resolving domain(s) that might appear on a blocklist, 
    //   i.e. exampleadultsite.com
    await checkContentFilters();
    
    // Check for split horizon by listening for each 
    // request and seeing if either: 
    //   a) the URL doesn't have a public suffix
    //   b) the URL has a public suffix but resolves to an RFC 1918 address
    let filter = {urls: ["<all_urls>"], types: ["main_frame"]};
    browser.webRequest.onBeforeRedirect.addListener(checkSplitHorizon, filter);
    browser.webRequest.onCompleted.addListener(checkSplitHorizon, filter);
  }
};


rollout.init();
